@@ -1,14 +1,15 @@
 import socket
 import json
+from pathlib import Path
 from datetime import datetime
 from .helpers import *
 from .status import status as sts
 
 """
 TODOS:
-- Send request params to app.
-- Add dynamic content-length and date headers to response.
 - Make query params accessible from app.
+- Default 404 error message does not work. Fix it.
+- Add error report.
 """
 
 # defaults
@@ -43,10 +44,9 @@ class Response:
     
     
     def get_full_res_text(self):
-        headers_as_string = ""
-        if self.__headers:
-            headers_as_string = "\r\n" + get_headers_as_string(self.__headers)
-        return f"{HTTP_VERSION} {self.__status} {self.__status_message}{headers_as_string}\r\n\r\n{self.__body}"
+        self.__headers["content-length"] = len(self.__body)
+        self.__headers["date"] = datetime.now()
+        return f"{HTTP_VERSION} {self.__status} {self.__status_message}\r\n{get_headers_as_string(self.__headers)}\r\n\r\n{self.__body}"
     
     
     def set_header(self, header, value=False):
@@ -62,6 +62,14 @@ class Response:
         self.__status_message = sts[str(self.__status)]
         self.__body = body
         
+    
+    def send_file(self, filename, status=200):
+        self.__status = status
+        self.__status_message = sts[str(self.__status)]
+        absolute_path = Path(filename).resolve()
+        with open(absolute_path, "r", encoding="utf-8") as f:
+            self.__body = f.read()
+        # need error handle here
 
 class Tea:
     def __init__(self):
@@ -81,11 +89,22 @@ class Tea:
         parsed_path = parse_path(path)
         self.__rules.append({ "method": "GET", "path": parsed_path, "callback": callback })
         
+        
     def post(self, path, callback):
         parsed_path = parse_path(path)
         self.__rules.append({ "method": "POST", "path": parsed_path, "callback": callback })
         
         
+    def put(self, path, callback):
+        parsed_path = parse_path(path)
+        self.__rules.append({ "method": "PUT", "path": parsed_path, "callback": callback })
+        
+        
+    def delete(self, path, callback):
+        parsed_path = parse_path(path)
+        self.__rules.append({ "method": "DELETE", "path": parsed_path, "callback": callback })
+        
+        
     def __handle_req(self, req, conn):
         parsed_req = parse_raw_http_req(req)
         
